<?php

namespace Modules\CategoryManager\Http\Controllers\Admin;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Modules\CategoryManager\Entities\Category;
use Illuminate\Support\Facades\DB;
use Modules\CategoryManager\Http\Requests\CategoryRequest;

/**
 * Categories Controller
 *
 * @property \Modules\CategoryManager\Entities\Category $Category
 *
 * @method \Modules\CategoryManager\Entities\Category[] index(Request $request), create, store(CategoryRequest $request), show(Category $Category), edit($id), update(CategoryRequest $request, $id), destroy($id)
 */
class CategoriesController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index(Request $request)
    {
        $allowed_columns = ['id', 'title', 'slug'];
        $sort = in_array($request->get('sort'), $allowed_columns) ? $request->get('sort') : 'lft';
        $order = $request->get('direction') === 'asc' ? 'asc' : 'asc';
        $categories = Category::status(request('status'))->categoryWise(request('category_id'))->filter(request('keyword'))->with('ancestors')->orderBy($sort, $order)->orderBy('title', 'desc')->paginate(config('get.ADMIN_PAGE_LIMIT'));
        $parentCategories = Category::where('status', 1)->whereNull('parent_id')->pluck('title','id');
        $parentCategories->prepend('Select Parent Category' , "");
        return view('categorymanager::Admin.categories.index', compact('categories','parentCategories'));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        $tree = Category::get()->where('status', 1)->toTree();
        $parentCategories[''] = 'Select Parent';
        $traverse = function ($categories, $prefix = '') use (&$traverse, &$parentCategories) {
            foreach ($categories as $category) {
                $parentCategories[$category->id] = $prefix.' '.$category->title;
                $traverse($category->children, $prefix.'_');
            }
        };
        $traverse($tree);
        return view('categorymanager::Admin.categories.createOrUpdate',compact('parentCategories'));
    }

    /**
     * Store a newly created resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function store(CategoryRequest $request)
    {
        try{
            Category::create($request->all());
        }
        catch (\Illuminate\Database\QueryException $e) {
            return back()->withError($e->getMessage())->withInput();
        }
        return redirect()->route('admin.categories')->with('success', 'Category has been saved Successfully');
    }

    /**
     * Show the specified resource.
     * @return Response
     */
    public function show($id)
    {
        $category = Category::find($id);
        return view('categorymanager::Admin.categories.show', compact('category'));
    }

    /**
     * Show the form for editing the specified resource.
     * @return Response
     */
    public function edit($id)
    {
        $category = Category::find($id);
        $tree = Category::get()->where('status', 1)->where('id','!=', $id)->toTree();
        $parentCategories[''] = 'Select Parent';
        $traverse = function ($categories, $prefix = '') use (&$traverse, &$parentCategories) {
            foreach ($categories as $category) {
                $parentCategories[$category->id] = $prefix.' '.$category->title;
                $traverse($category->children, $prefix.'_');
            }
        };
        $traverse($tree);
        return view('categorymanager::Admin.categories.createOrUpdate', compact('category','parentCategories'));
    }

    /**
     * Update the specified resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function update(CategoryRequest $request, $id)
    {
        try{
            $category = Category::find($id);
            $category->fill($request->all());
            $category->save();
          }
          catch (\Illuminate\Database\QueryException $e) {
              return back()->withError($e->getMessage())->withInput();
          }
        return redirect()->route('admin.categories')->with('success', 'Category has been updated Successfully');
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy($id)
    {
        DB::beginTransaction();
        $category = Category::where('id', '=', $id)->first();
       // dd($category->products->count());
        
       try{
        $category->delete();
        DB::commit();
        $responce =  ['status' => true,'message' => 'This category has been deleted Successfully!','data' => $category];
    }
    catch (\Exception $e)
    {
        DB::rollBack();
        $responce =  ['status' => false,'message' => $e->getMessage()];
    }
    return $responce;

    }
}
